# libraries
library(raster)
library(sp)
library(ape)
library(phytools)
library(dplyr)

# functions to streamline
richnessLatitudeMeans <- function(richness_latitude, pa_t1, t_1, agg_lat){
  if(ncol(pa_t1)>3){
    t1_richness <- data.frame(agg_lat$y,rowSums(agg_lat[,2:ncol(agg_lat)], na.rm=T))
    colnames(t1_richness) <- c("latitude",paste(t_1))
    richness_latitude <- left_join(richness_latitude, t1_richness, by="latitude")
  } else {
    ti_richness <- data.frame(agg_lat$y, rep(1, nrow(agg_lat)))
    colnames(ti_richness) <- c("latitude",paste(t_1))
    richness_latitude <- left_join(richness_latitude, ti_richness, by="latitude")
  }
  return(richness_latitude)
}

richnessTemperatureMeans <- function(richness_temperature, pa_t1, t_1, agg_temp){
  if(ncol(pa_t1)>3){
    t1_richness <- data.frame(agg_temp$temp,rowSums(agg_temp[,2:ncol(agg_temp)], na.rm=T))
    colnames(t1_richness) <- c("temperature",paste(t_1))
    richness_temperature <- left_join(richness_temperature, t1_richness, by="temperature")
  } else {
    ti_richness <- data.frame(agg_temp$temp, rep(1, nrow(agg_temp)))
    colnames(ti_richness) <- c("temperature",paste(t_1))
    richness_temperature <- left_join(richness_temperature, ti_richness, by="temperature")
  }
  return(richness_temperature)
}

paDiversity <- function(pa_t1){
  
  if(ncol(pa_t1)==3){ 
    t_1_diversity <- aggregate(pa_t1, by = list(pa_t1$y), FUN = function(x){ifelse(any(x==1), 1, 0)})[,4]
  } else {
    agg <- aggregate(pa_t1, by = list(pa_t1$y), FUN = function(x){ifelse(any(x==1), 1, 0)}) 
    t_1_diversity <- rowSums(agg[, 4:ncol(agg)])
  }
}

speciationLatitudeMeans <- function(speciation_latitude, new_species, pa_t2, agg_lat){
  if(length(new_species)>0){
    # find which latitude dey at
    if(length(new_species) == 1){
      new_species_t2_pa <- agg_lat[, c("y", new_species)]
      new_species_t2_pa[,2] <- new_species_t2_pa[,2]
      new_species_t2_pa[which(is.na(new_species_t2_pa[,2])),2] <- 0
      colnames(new_species_t2_pa) <- c("latitude", paste(t_2))
    } else {
      agg_lat <- agg_lat[, c("y", new_species)]
      new_species_t2_pa <- data.frame(agg_lat$y,rowSums(agg_lat[,2:ncol(agg_lat)], na.rm=T))
      new_species_t2_pa[which(is.na(new_species_t2_pa[,2])),2] <- 0
      colnames(new_species_t2_pa) <- c("latitude", paste(t_2))
    }
    speciation_latitude <- left_join(speciation_latitude, new_species_t2_pa, by="latitude")
  } else {
    new_species_t2_pa <- data.frame(agg_lat$y,rep(0, nrow(agg_lat)))
    new_species_t2_pa[which(is.na(new_species_t2_pa[,2])),2] <- 0
    colnames(new_species_t2_pa) <- c("latitude", paste(t_2))
    speciation_latitude <- left_join(speciation_latitude, new_species_t2_pa, by="latitude")
  }
  return(speciation_latitude)
}

speciationTemperatureMeans <- function(speciation_latitude, new_species, pa_t2, agg_temp){
  if(length(new_species)>0){
    # find which latitude dey at
    if(length(new_species) == 1){
      new_species_t2_pa <- agg_temp[, c("temp",new_species)]
      new_species_t2_pa[which(is.na(new_species_t2_pa[,2])),2] <- 0
      colnames(new_species_t2_pa) <- c("temperature", paste(t_2))
    } else {
      agg_temp <- agg_temp[, c("temp",new_species)]
      new_species_t2_pa <- data.frame(agg_temp$temp,rowSums(agg_temp[,2:ncol(agg_temp)], na.rm=T))
      new_species_t2_pa[which(is.na(new_species_t2_pa[,2])),2] <- 0
      colnames(new_species_t2_pa) <- c("temperature", paste(t_2))
    }
    speciation_temperature <- left_join(speciation_temperature, new_species_t2_pa, by="temperature")
  } else {
    new_species_t2_pa <- data.frame(agg_temp$temp,rep(0, nrow(agg_temp)))
    new_species_t2_pa[which(is.na(new_species_t2_pa[,2])),2] <- 0
    colnames(new_species_t2_pa) <- c("temperature", paste(t_2))
    speciation_temperature <- left_join(speciation_temperature, new_species_t2_pa, by="temperature")
  }
  return(speciation_temperature)
}

extinctionLatitudeMeans <- function(extinction_latitude, new_dead_species, pa_t1, agg_lat){
  if(length(new_dead_species)>0){
    # find which latitude dey at
    if(length(new_dead_species) == 1){
      # needs to look at t1 occurence because the species is dead in t2 (opposite of speciation)
      new_dead_species_t1_pa <- try(pa_t1[, c("y", new_dead_species)], silent = T)
      if(class(new_dead_species_t1_pa) == "try-error"){
        # if the species arises and goes extinct in the same time step we actually don't have a record of where it was at all (i think)
        # pretend it didn';'t exist for now
        new_dead_species_t1_pa  <- data.frame(agg_lat$y,rep(0, nrow(agg_lat)))
        new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
        colnames(new_dead_species_t1_pa) <- c("latitude", paste(t_2))
        extinction_latitude <- left_join(extinction_latitude, new_dead_species_t1_pa, by="latitude")
      } else {
        new_dead_species_t1_pa <- agg_lat[,c("y",new_dead_species)]
        new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
        colnames(new_dead_species_t1_pa) <- c("latitude", paste(t_2))
      }
      
    } else {
      new_dead_species_t1_pa <- try(pa_t1[, c("y", new_dead_species)], silent = T)
      if(class(new_dead_species_t1_pa) == "try-error"){
        if(any(new_dead_species %in% colnames(pa_t1))){
          new_dead_species <- new_dead_species[which(new_dead_species %in% colnames(pa_t1))]
          if(length(new_dead_species) == 1){
            new_dead_species_t1_pa <- agg_lat[,c("y",new_dead_species)]
            new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
            colnames(new_dead_species_t1_pa) <- c("latitude", paste(t_2))
          } else {
            agg_lat <- agg_lat[,c("y",new_dead_species)]
            new_dead_species_t1_pa <- data.frame(agg_lat$y,rowSums(agg_lat[,2:ncol(agg_lat)], na.rm=T))
            new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
            colnames(new_dead_species_t1_pa) <- c("latitude", paste(t_2))
          }
          
          
        }
      } else {
        agg_lat <- agg_lat[,c("y",new_dead_species)]
        new_dead_species_t1_pa <- data.frame(agg_lat$y,rowSums(agg_lat[,2:ncol(agg_lat)], na.rm=T))
        new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
        colnames(new_dead_species_t1_pa) <- c("latitude", paste(t_2))
      }
    }
    extinction_latitude <- left_join(extinction_latitude, new_dead_species_t1_pa, by="latitude")
  } else {
    new_dead_species_t1_pa  <- data.frame(agg_lat$y,rep(0, nrow(agg_lat)))
    new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
    colnames(new_dead_species_t1_pa) <- c("latitude", paste(t_2))
    extinction_latitude <- left_join(extinction_latitude, new_dead_species_t1_pa, by="latitude")
  }
 return(extinction_latitude) 
}

extinctionTemperatureMeans <- function(extinction_temperature, new_dead_species, pa_t1, agg_temp){
  if(length(new_dead_species)>0){
    # find which temperature dey at
    if(length(new_dead_species) == 1){
      # needs to look at t1 occurence because the species is dead in t2 (opposite of speciation)
      new_dead_species_t1_pa <- try(pa_t1[, c("y", new_dead_species)], silent = T)
      if(class(new_dead_species_t1_pa) == "try-error"){
        # if the species arises and goes extinct in the same time step we actually don't have a record of where it was at all (i think)
        # pretend it didn';'t exist for now
        new_dead_species_t1_pa  <- data.frame(agg_temp$temp,rep(0, nrow(agg_temp)))
        new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
        colnames(new_dead_species_t1_pa) <- c("temperature", paste(t_2))
        extinction_temperature <- left_join(extinction_temperature, new_dead_species_t1_pa, by="temperature")
      } else {
        new_dead_species_t1_pa <- agg_temp[,c("temp",new_dead_species)]
        new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
        colnames(new_dead_species_t1_pa) <- c("temperature", paste(t_2))
      }
      
    } else {
      new_dead_species_t1_pa <- try(pa_t1[, c("temp", new_dead_species)], silent = T)
      if(class(new_dead_species_t1_pa) == "try-error"){
        if(any(new_dead_species %in% colnames(pa_t1))){
          new_dead_species <- new_dead_species[which(new_dead_species %in% colnames(pa_t1))]
          if(length(new_dead_species) == 1){
            new_dead_species_t1_pa <- agg_temp[,c("temp",new_dead_species)]
            new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
            colnames(new_dead_species_t1_pa) <- c("temperature", paste(t_2))
          } else {
            agg_temp <- agg_temp[,c("temp",new_dead_species)]
            new_dead_species_t1_pa <- data.frame(agg_temp$temp,rowSums(agg_temp[,2:ncol(agg_temp)], na.rm=T))
            new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
            colnames(new_dead_species_t1_pa) <- c("temperature", paste(t_2))
          }
          
          
        }
      } else {
        agg_temp <- agg_temp[,c("y",new_dead_species)]
        new_dead_species_t1_pa <- data.frame(agg_temp$temp,rowSums(agg_temp[,2:ncol(agg_temp)], na.rm=T))
        new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
        colnames(new_dead_species_t1_pa) <- c("temperature", paste(t_2))
      }
    }
    extinction_temperature <- left_join(extinction_temperature, new_dead_species_t1_pa, by="temperature")
  } else {
    new_dead_species_t1_pa  <- data.frame(agg_temp$temp,rep(0, nrow(agg_temp)))
    new_dead_species_t1_pa[which(is.na(new_dead_species_t1_pa[,2])),2] <- 0
    colnames(new_dead_species_t1_pa) <- c("temperature", paste(t_2))
    extinction_temperature <- left_join(extinction_temperature, new_dead_species_t1_pa, by="temperature")
  }
  return(extinction_temperature) 
}

# choose model to begin
#setwd("//ites-fedata.ethz.ch/Simulation/Alex/BIGEST/EER/m0-m3_Scotese_Behr220_parameter_exploration_v5")

landscape <- readRDS("//ites-fedata.ethz.ch/Simulation/Alex/input/Scotese_Behrmann_world_220x220km/landscapes.rds")

for(m in 1:4){
  
  if(m == 1){model="m0"}
  if(m == 2){model="m1"}
  if(m == 3){model="m2"}
  if(m == 4){model="m3"}
  
  print(paste(model, model, model, model, sep=" ::: "))
  
  setwd(paste0("output/",model))
  
  # list the files
  m_files <- list.files(pattern=model)
  m_files <- m_files[order(as.numeric(sapply(m_files, FUN=function(m)strsplit(m, "_")[[1]][6])))]
  
  DRTT_list                <- vector("list", 100)
  richness_latitude_list   <- vector("list", 100)
  speciation_latitude_list <- vector("list", 100)
  extinction_latitude_list <- vector("list", 100)
  richness_temperature_list   <- vector("list", 100)
  speciation_temperature_list <- vector("list", 100)
  extinction_temperature_list <- vector("list", 100)
  
  for(i in 1:100){
    
    print(paste(model, "simulation", i, sep=" "))
    setwd(file.path(m_files[i]))
    
    # see if the model finsihed on the final time step (0)
    setwd(file.path("richness"))
    files <- list.files()
    setwd("../")
    most_recent <- min(as.numeric(sapply(files, FUN=function(x){strsplit(strsplit(x, "_")[[1]][3], ".rds")[[1]]})))
    if(most_recent != 0){next}
   
    ###~~~~~~~~~~~~~~~~~ TEMPORAL VARIATION IN DIVERSIFICATION RATES ~~~~~~~~~~~~~~~~#
    
    sim <- readRDS("sgen3sis.rds")
    
    # look at the overall speciation rate thorugh time # N. speciation events divdided by number of extant lineages in preceding time step
    SRTT  <- sim$summary$phylo_summary[2:nrow(sim$summary$phylo_summary),3] / sim$summary$phylo_summary[1:nrow(sim$summary$phylo_summary)-1,2] 
    # look at the overall extinction rate thorugh time # N. extinction events divdided by number of extant lineages in preceding time step
    ERTT  <- sim$summary$phylo_summary[2:nrow(sim$summary$phylo_summary),4] / sim$summary$phylo_summary[1:nrow(sim$summary$phylo_summary)-1,2]
    # combine to get diversification rate
    DRTT <- SRTT-ERTT
    DRTT <- data.frame(DR=DRTT,SR=SRTT, ER=ERTT, time=seq(from=65.16667, to=0, by=-0.16667))
    DRTT_list[[i]] <-DRTT

    
    ###~~~~~~ LATITUDINAL AND TEMPERATUTE VARIATION IN DIVERSITY AND SPECIATION/EXTINCTION EVENTS ~~~~~~~#

    t_og <- 390
    # get reference p/a matrix to set up data frame
    setwd("pa_matrices")
    pa_t2 <- read.table(paste0("species_pa_", t_og, ".txt"), header=T)

    # add the temperature columns from the landscape to match with
    temp_df <- merge(landscape$temp[,c(1:2, (t_og+3))], pa_t2, by=c("x","y"))
    temp_df[,3] <- round(temp_df[,3])
    colnames(temp_df)[3] <- "temp"
    
    # aggregate at latitudinal bands and temperature bands
    agg_lat <- aggregate(. ~ y,data = pa[,2:ncol(pa)], FUN = function(x){ifelse(any(x==1), 1, 0)})
    agg_temp <- aggregate(. ~ temp,  data= temp_df[,3:ncol(temp_df)], FUN = function(x){ifelse(any(x==1), 1, 0)})
    
    #create data frames to store rates
    speciation_latitude <- data.frame(latitude = agg_lat$y)
    extinction_latitude <- data.frame(latitude = agg_lat$y)
    richness_latitude   <- data.frame(latitude = agg_lat$y)
    speciation_temperature <- data.frame(temperature = c(-38:27))
    extinction_temperature <- data.frame(temperature = c(-38:27))
    richness_temperature   <- data.frame(temperature = c(-38:27))
    
    
    # for each time step
    for(t_2 in 389:0){
      #print(t_2)
      # read in the p/a matrices of time step 1 and time step 2
      t_1 <- t_2+1
      pa_t1 <- pa_t2 # older time - so it just the younger time of the previous iteration
      pa_t2 <- read.table(paste0("species_pa_", t_2, ".txt"), header=T) # younger time
      
      temp_t1 <- merge(landscape$temp[,c(1:2, (t_1+3))], pa_t1, by=c("x", "y"))
      temp_t1[,3] <- round(temp_t1[,3])
      colnames(temp_t1)[3] <- "temp"
      
      temp_t2 <- merge(landscape$temp[,c(1:2, (t_2+3))], pa_t2, by=c("x", "y"))
      temp_t2[,3] <- round(temp_t2[,3])
      colnames(temp_t2)[3] <- "temp"
      
     
      
      # aggregate by latitude and temperature
      agg_lat <- aggregate(. ~ y,data = pa_t2[,2:ncol(pa_t2)], FUN = function(x){ifelse(any(x==1), 1, 0)})
      agg_temp <- aggregate(. ~ temp,  data= temp_t2[,3:ncol(temp_t2)], FUN = function(x){ifelse(any(x==1), 1, 0)})
      

      # get richness (just rowSums of the aggregated data)
      richness_latitude <- richnessLatitudeMeans(richness_latitude, pa_t2, t_2, agg_lat)
      richness_temperature <- richnessTemperatureMeans(richness_temperature, pa_t2, t_2, agg_temp)
      
      # species names
      species_t1 <- colnames(pa_t1)[3:ncol(pa_t1)]
      species_t2 <- colnames(pa_t2)[3:ncol(pa_t2)]
      
      # extinct species names
      dead_species_t1 <- colnames(pa_t1)[colSums(pa_t1)==0]
      dead_species_t2 <- colnames(pa_t2)[colSums(pa_t2)==0]
      
      
      # species in t2 but not t1 are new (or newly dead)
      new_species <- species_t2[which(!species_t2 %in% species_t1)]
      new_dead_species <- dead_species_t2[which(!dead_species_t2 %in% dead_species_t1)]
      
      # speciation events by latitude
      speciation_latitude    <- speciationLatitudeMeans(speciation_latitude, new_species, pa_t2, agg_lat)
      speciation_temperature <- speciationTemperatureMeans(speciation_temperature, new_species, pa_t2, agg_temp)
      
      # extinction events by latitude
      extinction_latitude    <- extinctionLatitudeMeans(extinction_latitude, new_dead_species, pa_t1, agg_lat)
      extinction_temperature <- extinctionTemperatureMeans(extinction_temperature, new_dead_species, pa_t1, agg_temp)
      
    }
    
    # store data
    speciation_latitude_list[[i]]  <-  speciation_latitude
    richness_latitude_list[[i]]       <- richness_latitude
    extinction_latitude_list[[i]] <-  extinction_latitude
    
    speciation_temperature_list[[i]]  <-  speciation_temperature
    richness_temperature_list[[i]]       <- richness_temperature
    extinction_temperature_list[[i]] <-  extinction_temperature
    
    # go back to home
    setwd("../../")
    
    # save each output (this way we don';'t lose any progress in case of crashes)
    saveRDS(list(
      DRTT_list,                          
      richness_latitude_list, 
      speciation_latitude_list,       
      extinction_latitude_list,
      richness_temperature_list, 
      speciation_temperature_list,       
      extinction_temperature_list
    ), file=paste0(model, "_DRTST.rds"))
    
    
  }
  
  saveRDS(list(
    DRTT_list,                          
    richness_latitude_list, 
    speciation_latitude_list,       
    extinction_latitude_list,
    richness_temperature_list, 
    speciation_temperature_list,       
    extinction_temperature_list
  ), file=paste0(model, "_DRTST.rds"))
}

